"""
-*- encoding: utf-8 -*-

ASRRunner
继承RunnerBase
专属pipeline为ASRPipeline
"""

from src.pipeline.ASRpipeline import ASRPipeline
from src.pipeline import PipelineFactory
from src.models import AudioBinary_data_list, AudioBinary_Config
from src.core.model_loader import ModelLoader
from src.config import DefaultConfig
from queue import Queue
import soundfile
import time
from typing import List, Optional
import uuid
from threading import Thread
from src.utils.mock_websocket import MockWebSocketClient as WebSocketClient
from .runner import RunnerBase

from src.utils.logger import get_module_logger

logger = get_module_logger(__name__)


OVAERWATCH = False

model_loader = ModelLoader()

class ASRRunner(RunnerBase):
    """
    运行器类
    负责管理资源和协调Pipeline的运行
    """
    class SenderAndReceiver:
        """
        对于单个pipeline的管理
        包含 发送者 和 接收者
        _sender: 发送者 唯一
        _receiver: 接收者 可以有多个
        _pipeline: 对应管道 唯一
        """
        def __init__(self, *args, **kwargs):
            # 可选传入参数，
            self._name: str = kwargs.get("name", "")
            self._sender: Optional[WebSocketClient] = kwargs.get("sender", None)
            self._receiver: List[WebSocketClient] = kwargs.get("receiver", [])

            # 资源
            self._audio_config: AudioBinary_Config = kwargs.get("audio_config", DefaultConfig.audio_config)
            self._models: dict = kwargs.get("models", None)
            self._audio_binary: AudioBinary_data_list = AudioBinary_data_list()
            # id唯一标识
            self._id: str = str(uuid.uuid4())
            # 输入队列
            self._input_queue: Queue = Queue()
            self._pipeline: Optional[ASRPipeline] = None

        def set_name(self, name: str):
            self._name = name

        def set_id(self, id: str):
            self._id = id

        def set_sender(self, sender: WebSocketClient):
            self._sender = sender

        def set_pipeline(self, pipeline: ASRPipeline):
            self._pipeline = pipeline
            config = {
                "audio_config": self._audio_config,
            }
            self._pipeline.set_config(config)
            self._pipeline.set_models(self._models)
            self._pipeline.set_audio_binary(self._audio_binary)
            self._pipeline.set_input_queue(self._input_queue)
            self._pipeline.set_callback(self.deal_message)
            self._pipeline.bake()

        def append_receiver(self, receiver: WebSocketClient):
            self._receiver.append(receiver)
        
        def delete_receiver(self, receiver: WebSocketClient):
            self._receiver.remove(receiver)

        def deal_message(self, message: str):
            self.broadcast(message)

        def broadcast(self, message: str):
            """
            广播发送给所有接收者
            """
            logger.info("[ASRRunner][SAR-%s]广播发送给所有接收者: %s", self._name, message)
            for receiver in self._receiver:
                receiver.send(message)

        def _run(self):
            """
            运行SAR
            """
            self._pipeline.run()
            while True:
                data = self._sender.recv()
                if data is None:
                    break
                logger.debug("[ASRRunner][SAR-%s]接收到的数据length: %s", self._name, len(data))
                self._input_queue.put(data)
            self.stop()

        def run(self):
            """
            运行SAR
            """
            self._thread = Thread(target=self._run, name=f"[ASRRunner]SAR-{self._name}")
            self._thread.daemon = True
            self._thread.start()
        
        def stop(self):
            """
            停止SAR
            """
            self._pipeline.stop()
            for ws in self._receiver:
                ws.close()
            self._sender.close()
        
    def __init__(self,*args,**kwargs):
        """
        """ 
        # 接收资源
        self._default_audio_config = kwargs.get("audio_config", DefaultConfig.audio_config)
        # self._audio_binary_list = args.get("audio_binary_list", None)
        self._default_models = kwargs.get("models", None)
        self._SAR_list: List[self.SenderAndReceiver] = []

    def set_default_config(self, *args, **kwargs):
        """
        设置配置
        """
        self._default_audio_config = kwargs.get("audio_config", self._default_audio_config)
        self._default_models = kwargs.get("models", self._default_models)

    def new_SAR(
            self, 
            ws: "WebSocketClient",
            name: str = "", 
            audio_config: "AudioBinary_Config" = None,
            models: dict = None
        ) -> uuid.UUID:
        """
        创建新的SAR SenderAndReceiver
        """
        if audio_config is None:
            audio_config = self._default_audio_config
        if models is None:
            models = self._default_models

        try:
            new_SAR = self.SenderAndReceiver(
                name=name,
                audio_config=audio_config,
                models=models
            )
            new_pipeline = ASRPipeline()
            new_SAR.set_pipeline(new_pipeline)
            # new_SAR.set_pipeline()
            logger.info("创建新的SAR: name %s, id %s", new_SAR._name, new_SAR._id)
            new_SAR.set_sender(ws)
            new_SAR.append_receiver(ws)
            new_SAR.run()
            self._SAR_list.append(new_SAR)
            return new_SAR._id
        except Exception as e:
            logger.error("创建管道失败: %s", e)
            return None

    def join_SAR(
            self,
            ws: "WebSocketClient",
            name: Optional[str] = None,
            id: Optional[str] = None,
        ) -> bool:
        """
        加入SAR的Receiver
        """
        # 使用next获取迭代器下一个元素，生成pipeline_list迭代器，按id停止
        if id:
            exist_pipeline = next((pipeline for pipeline in self._SAR_list if pipeline._id == id), None)
        if name:
            exist_pipeline = next((pipeline for pipeline in self._SAR_list if pipeline._name == name), None)
        if exist_pipeline:
            exist_pipeline.append_receiver(ws)
            return True
        return False
  
    def __del__(self) -> None:
        """
        析构函数
        """
        for sar in self._SAR_list:
            sar.stop()
