import queue

from src.utils.logger import get_module_logger

logger = get_module_logger(__name__)

class MockWebSocketClient:
    """A mock WebSocket client to simulate a connection for testing."""

    def __init__(self):
        self.sent_messages = []
        self._is_closed = False
        self.receive_queue = queue.Queue()

    def send(self, message: dict):
        """Simulates sending a message (which is a dict)."""
        if self._is_closed:
            print("Warning: sending message on a closed websocket")
            return
        self.sent_messages.append(message)
        print(f"Mock WS received: {message}")

    def recv(self):
        """Simulates receiving data from the WebSocket."""
        if self._is_closed:
            return None
        try:
            # Block until data is available, with a timeout to prevent hanging.
            data = self.receive_queue.get(timeout=10)
            if data is None:
                self._is_closed = True
            return data
        except queue.Empty:
            print("Mock WS recv timeout")
            self._is_closed = True
            return None

    def close(self):
        """Simulates closing the WebSocket connection."""
        if not self._is_closed:
            # Put None to unblock any waiting recv call
            self.receive_queue.put(None)
            self._is_closed = True  
        print("Mock WS closed")

    def put_for_recv(self, data):
        """Puts data into the receive queue for the `recv` method to consume."""
        if data is None:
            return
        logger.debug("Mock WS put_for_recv length: %s", len(data))
        self.receive_queue.put(data)

    @property
    def is_closed(self):
        return self._is_closed